package	jp.co.sds_corp.ratemixer.fx.filter;

import java.math.BigDecimal;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* 為替レートをBid-Askの差でフィルタリングします。
XMLは以下の形式です。
<PRE>
&lt;filter class="jp.co.sds_corp.ratemixer.fx.filter.RangeFilter"&gt;
    &lt;target ccy1="USD" ccy2="JPY" term="SP"/&gt;
    &lt;target ccy1="EUR" ccy2="JPY" exclude="true"/&gt;
    &lt;pips&gt;10&lt;/pips&gt;
&lt;/filter&gt;
</PRE>
*/
public class SplitFilter extends FxFilter
{
	private static final long serialVersionUID = 1L;
	BigDecimal	pips;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		pips = new BigDecimal( config.get( "pips" ) );
	}

	/**
	* レートのフィルタリングを行なう
	* @param	ev		レート変更イベント
	*/
	public boolean isValid( ReceiveEvent ev ) throws Exception {
		FxRate rate = (FxRate )ev.getData();
		if ( rate == null || rate.getBid() == null || rate.getAsk() == null ) {
			return false;
		}
		CcyPair pair = rate.getCcyPair();
		BigDecimal dif = rate.getBid().subtract( rate.getAsk() ).abs()
				.multiply( new BigDecimal(
							Math.pow( 10.0, (double )pair.getPoint() ) ));
		if ( dif.compareTo( pips ) > 0 ) {
			return false;
		}
		return true;
	}

	/**
	* 最大値(pips)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPips( BigDecimal var ) {
		this.pips = var ;
	}
	/**
	* 最大値(pips)を取得します。
	* @return	pipsの内容
	*/
	public BigDecimal getPips() {
		return pips;
	}

}
